package be.iminds.ilabt.jfed.espec.filefetcher;

import be.iminds.ilabt.jfed.espec.bundle.ESpecBundle;
import be.iminds.ilabt.jfed.espec.model.FileSource;
import be.iminds.ilabt.jfed.espec.model.GeneratedRandomFileSource;
import be.iminds.ilabt.jfed.util.IOUtils;

import javax.annotation.Nonnull;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.nio.charset.StandardCharsets;

/**
 * FileFetcher
 *
 * Fetches file content from whatever source is used in an experimentSpecification
 * See FileSource.SourceType for types
 *
 */
public class FileFetcher {
    @Nonnull private final FileSource fileSource;
    @Nonnull private final ESpecBundle bundle;

    public FileFetcher(@Nonnull FileSource fileSource, @Nonnull ESpecBundle bundle) {
        this.fileSource = fileSource;
        this.bundle = bundle;
    }

    @Nonnull
    public FileSource getFileSource() {
        return fileSource;
    }

    @Nonnull
    public ESpecBundle getBundle() {
        return bundle;
    }

    public boolean isFast() {
        switch (fileSource.getType()) {
            case BUNDLED: return true;
            case DIRECT: return true;
            case DOWNLOAD: return false;
            case GIT: return false;
            case GITHUB: return false;
            case GENERATED: return true;
            default: throw new IllegalStateException("Unsupported FileSource type: "+fileSource.getType().name());
        }
    }

    @Nonnull
    public byte[] fetchBytes() throws IOException {
        switch (fileSource.getType()) {
            case BUNDLED: return fetchBundled();
            case DIRECT: return fetchDirect();
            case DOWNLOAD: return fetchDownload();
            case GIT: return fetchGit();
            case GITHUB: return fetchGithub();
            case GENERATED: {
                switch (fileSource.getValue().toLowerCase()) {
                    case "random" : {
                        if (!(fileSource instanceof GeneratedRandomFileSource))
                            throw new IllegalStateException("Internal bug: random fileSource should be " +
                                    "instance of GeneratedRandomFileSource, not "+fileSource.getClass().getName());
                        return generateRandom((GeneratedRandomFileSource) fileSource);
                    }
                    default: throw new IllegalStateException("fetchBytes should not be called for GENERATED "+fileSource.getValue());
                }
            }
            default: throw new IllegalStateException("Unsupported FileSource type: "+fileSource.getType().name());
        }
    }

    public static byte[] generateRandom(GeneratedRandomFileSource fileSource) {
        String format = fileSource.getFormat();
        int len = fileSource.getLength();

        switch (format.trim().toLowerCase()) {
            case "password":
                return RandomDataGenerator.generatePassword(len).getBytes(StandardCharsets.UTF_8);
            case "binary":
                return RandomDataGenerator.generateBinary(len);
            case "base64":
                return RandomDataGenerator.generateBase64(len).getBytes(StandardCharsets.UTF_8);
            case "alphanum":
                return RandomDataGenerator.generateAlphaNum(len).getBytes(StandardCharsets.UTF_8);
            default:
                throw new IllegalStateException("generate:random does not support format \""+format+"\"");
        }
    }


    @Nonnull
    private byte[] fetchBundled() throws IOException {
        return bundle.getFileContent(fileSource.getValue());
    }

    @Nonnull
    private byte[] fetchDirect() {
        return fileSource.getValue().getBytes(StandardCharsets.UTF_8);
    }

    @Nonnull
    private byte[] fetchDownload() {
        try {
            URL url = new URL(fileSource.getValue());
            return IOUtils.urlToByteArray(url);
        } catch (MalformedURLException e) {
            throw new RuntimeException("Invalid URL", e);
        } catch (IOException e) {
            throw new RuntimeException("Error reading URL", e);
        }
    }

    @Nonnull
    private byte[] fetchGit() {
        //TODO
        throw new RuntimeException("Not yet implemented");
    }

    @Nonnull
    private byte[] fetchGithub() {
        //TODO
        throw new RuntimeException("Not yet implemented");
    }
}
